'use strict'

const env = require("./environment")
const fs = require("fs")

// Method to validate url

const validURL = (str) => {
    var pattern = new RegExp('^(https?:\\/\\/)?'+ // protocol
      '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|'+ // domain name
      '((\\d{1,3}\\.){3}\\d{1,3}))'+ // OR ip (v4) address
      '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*'+ // port and path
      '(\\?[;&a-z\\d%_.~+=-]*)?'+ // query string
      '(\\#[-a-z\\d_]*)?$','i'); // fragment locator
    return !!pattern.test(str);
}

// Method to create picture directory
function createFolderFromURL (url) {
    if (!validURL(url)) {
        throw "URL is not valid"
    }
    url = url.replace(/^https?\:\/\//i, "");
    let splitAddress = url.split(/\//)
    splitAddress.unshift(env.screenshotsFolder)
    let address = ""
    for (let index in splitAddress) {
        address += splitAddress[index] + "/"
        if (!fs.existsSync(address)) {
            fs.mkdirSync(address);
        }
    }
    return address
}

module.exports = {
    screenshotDir: createFolderFromURL,
    reload: async (page) => {
        await page.goto(page.url());
    },
    goto: async (page, pageUrl) => {
        console.log("Page: " + pageUrl)
        try {
            await page.goto(pageUrl, {timeout: 0})
        } catch(e) {
            throw "Error when trying to go to page: " + e
        }
    },
    timeout: (ms) => {
        return new Promise(resolve => setTimeout(resolve, ms));
    },
    scrollPage: async (page) => {
        if (env.autoScroll === true) { 
            console.log('Scrolling through page...');
            try {
                // DOWN
                await page.evaluate(() => {
                    return new Promise((resolve, reject) => {
                        var totalHeight = 0;
                        var timer = setInterval(() => {
                            var scrollHeight = document.body.scrollHeight / 50;
                            window.scrollBy(0, scrollHeight);
                            totalHeight += scrollHeight;
                            if(totalHeight >= document.body.scrollHeight){
                                clearInterval(timer);
                                resolve();
                            }
                        }, 10);
                    })
                });
                console.log("Scrolling finished!")
                console.log("Waiting images loading...")
                await module.exports.timeout(env.imgLoad)
            } catch (e) {
                console.error(e)
                throw "Error when trying to scroll the page: " + e;
            }
        } else {
            console.log("Skipping scroll")
        }
    }
}